import { isPlatformBrowser, CommonModule } from '@angular/common';
import { Directive, Inject, PLATFORM_ID, Renderer2, ElementRef, Input, NgModule } from '@angular/core';
import { coerceBooleanProperty } from '@angular/cdk/coercion';

/*!
 * Clamp.js 0.5.1
 *
 * Copyright 2011-2013, Joseph Schmitt http://joe.sh
 * Released under the WTFPL license
 * http://sam.zoy.org/wtfpl/
 */
function createClampElement(element, options) {
    options = options || {};
    let win = window, opt = {
        clamp: options.clamp || 2,
        useNativeClamp: typeof (options.useNativeClamp) != 'undefined' ? options.useNativeClamp : true,
        splitOnChars: options.splitOnChars || ['.', '-', '–', '—', ' '],
        animate: options.animate || false,
        truncationChar: options.truncationChar || '…',
        truncationHTML: options.truncationHTML
    }, sty = element.style, originalText = element.innerHTML, supportsNativeClamp = typeof (element.style.webkitLineClamp) != 'undefined', clampValue = opt.clamp, isCSSValue = clampValue.indexOf && (clampValue.indexOf('px') > -1 || clampValue.indexOf('em') > -1), truncationHTMLContainer;
    if (opt.truncationHTML) {
        truncationHTMLContainer = document.createElement('span');
        truncationHTMLContainer.innerHTML = opt.truncationHTML;
    }
    function computeStyle(elem, prop) {
        if (!win.getComputedStyle) {
            win.getComputedStyle = function (el, pseudo) {
                this.el = el;
                this.getPropertyValue = function (prop) {
                    var re = /(\-([a-z]){1})/g;
                    if (prop == 'float')
                        prop = 'styleFloat';
                    if (re.test(prop)) {
                        prop = prop.replace(re, function () {
                            return arguments[2].toUpperCase();
                        });
                    }
                    return el.currentStyle && el.currentStyle[prop] ? el.currentStyle[prop] : null;
                };
                return this;
            };
        }
        return win.getComputedStyle(elem, null).getPropertyValue(prop);
    }
    function getMaxLines(height) {
        var availHeight = height || element.clientHeight, lineHeight = getLineHeight(element);
        return Math.max(Math.floor(availHeight / lineHeight), 0);
    }
    function getMaxHeight(clmp) {
        let lineHeight = getLineHeight(element);
        return lineHeight * clmp;
    }
    function getLineHeight(elem) {
        let lh = computeStyle(elem, 'line-height');
        if (lh == 'normal') {
            lh = parseInt(computeStyle(elem, 'font-size')) * 1.2;
        }
        return parseInt(lh);
    }
    var splitOnChars = opt.splitOnChars.slice(0), splitChar = splitOnChars[0], chunks, lastChunk;
    function getLastChild(elem) {
        if (elem.lastChild.children && elem.lastChild.children.length > 0) {
            return getLastChild(Array.prototype.slice.call(elem.children).pop());
        }
        else if (!elem.lastChild || !elem.lastChild.nodeValue || elem.lastChild.nodeValue == '' || elem.lastChild.nodeValue == opt.truncationChar) {
            elem.lastChild.parentNode.removeChild(elem.lastChild);
            return getLastChild(element);
        }
        else {
            return elem.lastChild;
        }
    }
    function truncate(target, maxHeight) {
        if (!maxHeight) {
            return;
        }
        function reset() {
            splitOnChars = opt.splitOnChars.slice(0);
            splitChar = splitOnChars[0];
            chunks = null;
            lastChunk = null;
        }
        var nodeValue = target.nodeValue.replace(opt.truncationChar, '');
        if (!chunks) {
            if (splitOnChars.length > 0) {
                splitChar = splitOnChars.shift();
            }
            else {
                splitChar = '';
            }
            chunks = nodeValue.split(splitChar);
        }
        if (chunks.length > 1) {
            lastChunk = chunks.pop();
            applyEllipsis(target, chunks.join(splitChar));
        }
        else {
            chunks = null;
        }
        if (truncationHTMLContainer) {
            target.nodeValue = target.nodeValue.replace(opt.truncationChar, '');
            element.innerHTML = target.nodeValue + ' ' + truncationHTMLContainer.innerHTML + opt.truncationChar;
        }
        if (chunks) {
            if (element.clientHeight <= maxHeight) {
                if (splitOnChars.length >= 0 && splitChar != '') {
                    applyEllipsis(target, chunks.join(splitChar) + splitChar + lastChunk);
                    chunks = null;
                }
                else {
                    return element.innerHTML;
                }
            }
        }
        else {
            if (splitChar == '') {
                applyEllipsis(target, '');
                target = getLastChild(element);
                reset();
            }
        }
        if (opt.animate) {
            setTimeout(() => {
                truncate(target, maxHeight);
            }, opt.animate === true ? 10 : opt.animate);
        }
        else {
            return truncate(target, maxHeight);
        }
    }
    function applyEllipsis(elem, str) {
        elem.nodeValue = str + opt.truncationChar;
    }
    if (clampValue == 'auto') {
        clampValue = getMaxLines();
    }
    else if (isCSSValue) {
        clampValue = getMaxLines(parseInt(clampValue));
    }
    let clampedText;
    if (supportsNativeClamp && opt.useNativeClamp) {
        sty.overflow = 'hidden';
        sty.textOverflow = 'ellipsis';
        sty.webkitBoxOrient = 'vertical';
        sty.display = '-webkit-box';
        sty.webkitLineClamp = clampValue;
        if (isCSSValue) {
            sty.height = opt.clamp + 'px';
        }
    }
    else {
        let height = getMaxHeight(clampValue);
        if (height <= element.clientHeight) {
            clampedText = truncate(getLastChild(element), height);
        }
    }
    return {
        'original': originalText,
        'clamped': clampedText
    };
}

class NtClampDirective {
    constructor(platformId, _renderer, _elementRef) {
        this.platformId = platformId;
        this._renderer = _renderer;
        this._elementRef = _elementRef;
        this._lines = 'auto';
        this._folded = true;
        this._clampable = false;
        this.truncation = '...';
    }
    set folded(value) {
        this._folded = coerceBooleanProperty(value);
        if (this._clamp) {
            if (this._folded) {
                this._clamping();
            }
            else {
                this._elementRef.nativeElement.innerHTML = this._clamp.original;
            }
        }
    }
    get folded() { return this._folded; }
    set clamp(value) { this._lines = value; }
    get clamp() { return this._lines; }
    get clampable() { return this._clampable; }
    ngAfterViewInit() {
        if (this.folded) {
            this._clamping();
        }
        if (isPlatformBrowser(this.platformId)) {
        }
    }
    ngOnChanges(changes) {
        if (changes.clamp && !changes.clamp.firstChange) {
            this._clamping();
        }
    }
    _clamping() {
        if (isPlatformBrowser(this.platformId)) {
            this._clamp = createClampElement(this._elementRef.nativeElement, {
                clamp: this.clamp,
                useNativeClamp: true,
                truncationChar: '&nbsp;',
                truncationHTML: this.truncation
            });
            if (this._clamp && this._clamp.clamped) {
                Promise.resolve().then(() => {
                    this._clampable = this._clamp !== this._clamp.clamped.replace(`${this.truncation}&nbsp;`, '');
                });
            }
        }
    }
}
NtClampDirective.decorators = [
    { type: Directive, args: [{
                selector: '[nt-clamp]',
                exportAs: 'clamp'
            },] }
];
NtClampDirective.ctorParameters = () => [
    { type: Object, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] },
    { type: Renderer2 },
    { type: ElementRef }
];
NtClampDirective.propDecorators = {
    truncation: [{ type: Input }],
    folded: [{ type: Input }],
    clamp: [{ type: Input }]
};

class NtClampModule {
}
NtClampModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                exports: [NtClampDirective],
                declarations: [NtClampDirective],
            },] }
];

export { NtClampDirective, NtClampModule };
//# sourceMappingURL=clamp.js.map
