import { InjectionToken, Injectable, Inject, Component, ViewEncapsulation, ChangeDetectionStrategy, ChangeDetectorRef, Input, NgModule } from '@angular/core';
import { OverlayModule } from '@angular/cdk/overlay';
import { CommonModule } from '@angular/common';
import { NavigationEnd, ActivatedRoute, Router, RouterModule } from '@angular/router';
import { NtOverlayModule } from '@ng-tangram/components/overlay';
import { Subject, BehaviorSubject, merge } from 'rxjs';
import { takeUntil, filter, switchMap } from 'rxjs/operators';
import { coerceBooleanProperty } from '@angular/cdk/coercion';

var NtNavUrlTreeStrategy;
(function (NtNavUrlTreeStrategy) {
    NtNavUrlTreeStrategy[NtNavUrlTreeStrategy["Integrity"] = 0] = "Integrity";
    NtNavUrlTreeStrategy[NtNavUrlTreeStrategy["OnlyParents"] = 1] = "OnlyParents";
})(NtNavUrlTreeStrategy || (NtNavUrlTreeStrategy = {}));
const NT_NAV_CONFIG = new InjectionToken('nt-nav-config');

const RESOLVE_TITLE_PLACEHOLDER = /^resolve::/;
function resolveTitle(snapshot, title) {
    var _a;
    if (RESOLVE_TITLE_PLACEHOLDER.test(title)) {
        const paths = title.replace(RESOLVE_TITLE_PLACEHOLDER, '').split('.');
        const key = paths.shift();
        try {
            if (key && ((_a = snapshot.data) === null || _a === void 0 ? void 0 : _a.hasOwnProperty(key))) {
                const data = snapshot.data[key];
                let path = paths.shift();
                let result = path ? data[path] : null;
                while (paths.length > 0 && result instanceof Object) {
                    path = paths.shift();
                    result = path ? result[path] : null;
                }
                return result;
            }
            return key || '';
        }
        catch (e) {
            console.error(e);
        }
    }
    return title;
}

const NT_NAV_MENUS = new InjectionToken('nt-nav-menus');
class NtNavigationAdapter {
    constructor(route, router, config, menus) {
        this.route = route;
        this.router = router;
        this.config = config;
        this.destroy = new Subject();
        this.menus = new BehaviorSubject([]);
        this.activatedUrlTree = new BehaviorSubject([]);
        this.activeUrl = new BehaviorSubject({});
        this.menus.next(this.fillNavMenuUrls(menus));
        router.events
            .pipe(takeUntil(this.destroy), filter(event => event instanceof NavigationEnd))
            .subscribe(() => this.buildUrlTree(this.route));
    }
    ngOnDestroy() {
        this.destroy.next();
        this.destroy.complete();
    }
    getRouteMenus() {
        return this.menus.asObservable();
    }
    fillNavMenuUrls(menus, parentPath) {
        menus.forEach(menu => {
            const commands = [this.config.baseUrl || './'];
            if (!!parentPath) {
                commands.push(parentPath);
            }
            if (menu.path) {
                commands.push(menu.path);
            }
            menu.commands = commands;
            if (Array.isArray(menu.children) && menu.children.length > 0) {
                this.fillNavMenuUrls(menu.children, menu.path);
            }
        });
        return menus;
    }
    buildUrlTree(route) {
        let url = [];
        const tree = [];
        while (route.firstChild) {
            route = route.firstChild;
            const snapshot = route.snapshot;
            if (snapshot.data && snapshot.data.title && snapshot.url.length > 0) {
                let data = snapshot.data, title;
                title = resolveTitle(snapshot, data.title);
                url = url.concat(snapshot.url.map(segment => segment.path));
                tree.push({ title, url, icon: data.icon });
            }
        }
        this.activeUrl.next(tree[tree.length - 1]);
        if (this.config.urlTreeStrategy === NtNavUrlTreeStrategy.OnlyParents) {
            tree.splice(tree.length - 1, 1);
        }
        this.activatedUrlTree.next(tree);
    }
}
NtNavigationAdapter.decorators = [
    { type: Injectable }
];
NtNavigationAdapter.ctorParameters = () => [
    { type: ActivatedRoute },
    { type: Router },
    { type: undefined, decorators: [{ type: Inject, args: [NT_NAV_CONFIG,] }] },
    { type: Array, decorators: [{ type: Inject, args: [NT_NAV_MENUS,] }] }
];

class NtNavBreadcrumbsComponent {
    constructor(navAdapter) {
        this.navAdapter = navAdapter;
        this.activeUrl$ = this.navAdapter.activeUrl.asObservable();
    }
}
NtNavBreadcrumbsComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-nav-breadcrumbs',
                template: "<ng-container *ngIf=\"activeUrl$ | async as activeUrl\">\n  <span class=\"nt-nav-activated-icon {{ activeUrl.icon }}\"></span>\n  <span class=\"nt-nav-activated-title\">{{ activeUrl.title }}</span>\n  <ul class=\"nt-nav-breadcrumbs-menu\">\n    <li *ngFor=\"let item of navAdapter.activatedUrlTree | async; let $index = index;\">\n      <a [routerLink]=\"['./'].concat(item.url)\">\n        {{ item.title }}\n      </a>\n    </li>\n  </ul>\n</ng-container>\n",
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    'class': 'nt-nav-breadcrumbs'
                }
            },] }
];
NtNavBreadcrumbsComponent.ctorParameters = () => [
    { type: NtNavigationAdapter }
];

class NtNavController {
    constructor(changeDetectorRef, navAdapter) {
        this.changeDetectorRef = changeDetectorRef;
        this.navAdapter = navAdapter;
        this.destroy = new Subject();
        this.routeMenusUpdate = new Subject();
        merge(this.routeMenusUpdate, navAdapter.getRouteMenus())
            .pipe(switchMap(() => navAdapter.getRouteMenus()), takeUntil(this.destroy))
            .subscribe(menus => {
            this._routeMenus = this._filterRouteMenus(menus);
            this._checkActivatedMenu();
            this.changeDetectorRef.markForCheck();
        });
        this.navAdapter.activatedUrlTree
            .pipe(takeUntil(this.destroy), filter(tree => tree.length > 0))
            .subscribe(() => {
            this._checkActivatedMenu();
            this.changeDetectorRef.markForCheck();
        });
    }
    _checkActivatedMenu() {
        const currentTree = this.navAdapter.activatedUrlTree.value;
        if (currentTree.length > 0) {
            const moduleName = currentTree[0].url.join('/');
            const activated = this._routeMenus.find(menu => menu.path === moduleName);
            if (activated) {
                activated.folded = true;
            }
        }
    }
    _filterRouteMenus(menus) {
        if (typeof this.filter !== 'function') {
            return menus;
        }
        menus = menus.filter(menu => {
            if (Array.isArray(menu.children) && menu.children.length > 0) {
                return true;
            }
            else {
                return this.filter(menu);
            }
        });
        menus = menus.map(menu => {
            const copiedMenu = Object.assign({}, menu);
            if (Array.isArray(copiedMenu.children)) {
                copiedMenu.children = copiedMenu.children.filter(child => this.filter(child));
            }
            return copiedMenu;
        });
        const result = [];
        menus.forEach(menu => {
            if (!menu.children || (Array.isArray(menu.children) && menu.children.length > 0)) {
                result.push(Object.assign({}, menu));
            }
        });
        return result;
    }
}

class NtNavSidebarComponent extends NtNavController {
    constructor(changeDetectorRef, navAdapter) {
        super(changeDetectorRef, navAdapter);
        this._dropdown = false;
        this._baseUrl = navAdapter.config.baseUrl || './';
    }
    get dropdown() { return this._dropdown; }
    set dropdown(value) {
        this._dropdown = coerceBooleanProperty(value);
    }
    get filter() { return this._filter; }
    set filter(value) {
        if (value) {
            this._filter = value;
            this.routeMenusUpdate.next();
        }
    }
}
NtNavSidebarComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-nav-sidebar',
                template: "\n<!-- <ng-content select=\"nt-nav-sidebar-logo\"></ng-content> -->\n<div class=\"nt-nav-sidebar-brand\">\n  <img class=\"nt-nav-sidebar-logo\" [src]=\"logo\">\n  <span class=\"nt-nav-sidebar-title\" *ngIf=\"!dropdown\">{{ title }}</span>\n</div>\n\n<ul class=\"nt-nav-sidebar-root-menu\">\n  <li *ngFor=\"let parent of _routeMenus\"\n    routerLinkActive=\"active\"\n    [class.folded]=\"parent.folded\"\n    [class.has-children]=\"parent.children\">\n\n    <ng-container *ngIf=\"!dropdown else dropdownMode\">\n      <a *ngIf=\"!parent.children else hasChildrenMenus\" [routerLink]=\"parent.commands\">\n        <i [class]=\"parent.data['icon']\"></i>\n        <span class=\"nt-nav-sidebar-title\">{{ parent.data['title'] }}</span>\n      </a>\n      <ng-template #hasChildrenMenus>\n        <a (click)=\"parent.folded = !parent.folded\">\n          <i [class]=\"parent.data['icon']\"></i>\n          <span class=\"nt-nav-sidebar-title\">{{ parent.data['title'] }}</span>\n        </a>\n        <ul class=\"nt-nav-sidebar-nested-menu\">\n          <li *ngFor=\"let child of parent.children\" routerLinkActive=\"active\">\n            <a [routerLink]=\"child.commands\">\n              <span class=\"nt-nav-sidebar-title\">{{ child.data['title'] }}</span>\n            </a>\n          </li>\n        </ul>\n      </ng-template>\n    </ng-container>\n\n    <ng-template #dropdownMode>\n      <ng-container *ngIf=\"!parent.children else hasChildrenMenus\">\n        <a #origin=\"cdkOverlayOrigin\"\n        cdkOverlayOrigin\n        (mouseenter)=\"overlay.markOpen()\"\n        (mouseleave)=\"overlay.markClose()\"\n        [routerLink]=\"parent.commands\">\n          <i [class]=\"parent.data['icon']\"></i>\n          <span class=\"nt-nav-sidebar-title\">{{ parent.data['title'] }}</span>\n\n          <nt-overlay\n            #overlay\n            overlayClass=\"nt-nav-dropdown-overlay\"\n            [origin]=\"origin\"\n            position=\"rightTop\"\n            [arrow]=\"true\"\n            (overlayEnter)=\"overlay.markOpen()\"\n            (overlayLeave)=\"overlay.markClose()\">\n            <div class=\"nt-nav-dropdown-pane\">\n              <div class=\"nt-nav-dropdown-title\">{{ parent.data['title'] }}</div>\n            </div>\n          </nt-overlay>\n        </a>\n      </ng-container>\n      <ng-template #hasChildrenMenus>\n        <a #origin=\"cdkOverlayOrigin\"\n          cdkOverlayOrigin\n          (mouseenter)=\"overlay.markOpen()\"\n          (mouseleave)=\"overlay.markClose()\">\n          <i [class]=\"parent.data['icon']\"></i>\n          <span class=\"nt-nav-sidebar-title\">{{ parent.data['title'] }}</span>\n          <nt-overlay\n            #overlay\n            overlayClass=\"nt-nav-dropdown-overlay\"\n            [origin]=\"origin\"\n            position=\"rightTop\"\n            [arrow]=\"true\"\n            (overlayEnter)=\"overlay.markOpen()\"\n            (overlayLeave)=\"overlay.markClose()\">\n            <div class=\"nt-nav-dropdown-pane\">\n              <div class=\"nt-nav-dropdown-title\">{{ parent.data['title'] }}</div>\n              <ul class=\"nt-nav-dropdown-menu\">\n                <li *ngFor=\"let child of parent.children\" routerLinkActive=\"active\">\n                  <a [routerLink]=\"child.commands\">\n                    <span>{{ child.data['title'] }}</span>\n                  </a>\n                </li>\n              </ul>\n            </div>\n          </nt-overlay>\n        </a>\n      </ng-template>\n    </ng-template>\n  </li>\n</ul>\n\n",
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    'class': 'nt-nav-sidebar',
                    '[class.dropdown-mode]': 'dropdown'
                }
            },] }
];
NtNavSidebarComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: NtNavigationAdapter }
];
NtNavSidebarComponent.propDecorators = {
    dropdown: [{ type: Input }],
    filter: [{ type: Input }],
    title: [{ type: Input }],
    logo: [{ type: Input }]
};

class NtNavTopbarComponent extends NtNavController {
    constructor(changeDetectorRef, navAdapter) {
        super(changeDetectorRef, navAdapter);
        this._baseUrl = navAdapter.config.baseUrl || './';
    }
    get filter() { return this._filter; }
    set filter(value) {
        if (value) {
            this._filter = value;
            this.routeMenusUpdate.next();
        }
    }
}
NtNavTopbarComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-nav-topbar',
                template: "<ul class=\"nt-nav-topbar-menu\">\n  <li *ngFor=\"let parent of _routeMenus\" routerLinkActive=\"active\" class=\"nt-nav-topbar-menu-item\">\n    <a [routerLink]=\"parent.commands\">\n      <span [class]=\"parent.data.icon\"></span>&nbsp;&nbsp;{{ parent.data.title }}\n    </a>\n  </li>\n</ul>\n",
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    'class': 'nt-nav-topbar'
                }
            },] }
];
NtNavTopbarComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: NtNavigationAdapter }
];
NtNavTopbarComponent.propDecorators = {
    filter: [{ type: Input }]
};

class NtNavModule {
    static forRoot(config) {
        return {
            ngModule: NtNavModule,
            providers: [
                { provide: NT_NAV_CONFIG, useValue: config }
            ]
        };
    }
}
NtNavModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    RouterModule,
                    OverlayModule,
                    NtOverlayModule,
                ],
                exports: [
                    NtNavSidebarComponent,
                    NtNavTopbarComponent,
                    NtNavBreadcrumbsComponent,
                ],
                declarations: [
                    NtNavSidebarComponent,
                    NtNavTopbarComponent,
                    NtNavBreadcrumbsComponent
                ],
                providers: [NtNavigationAdapter]
            },] }
];

export { NT_NAV_CONFIG, NT_NAV_MENUS, NtNavModule, NtNavSidebarComponent, NtNavTopbarComponent, NtNavUrlTreeStrategy, NtNavigationAdapter, resolveTitle, NtNavController as ɵa, NtNavBreadcrumbsComponent as ɵb };
//# sourceMappingURL=navigation.js.map
