import { Observable, Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import { Component, HostListener, OnDestroy, OnInit } from '@angular/core';
import { NtDropdownComponent } from '@ng-tangram/components/dropdown';
import { NtSwitchChange } from '@ng-tangram/components/switch';
import { Select, Store } from '@ngxs/store';

import { UIActions, UIFontSize, UISidebarMode } from './user-interface.actions';
import { UIState, UIStateModel } from './user-interface.state';

function getFontSizeObject(keys: string[]) {
  const obj: { [key: string]: number } = {};
  keys.forEach(key => {
    obj[key] = UIFontSize[key];
  });
  return obj;
}

@Component({
  selector: 'app-customization',
  templateUrl: 'customization.component.html',
  styleUrls: ['customization.component.scss']
})
export class CustomizationComponent implements OnInit, OnDestroy {

  private readonly _destroy = new Subject();

  @Select(UIState) ui$: Observable<UIStateModel>;

  @Select(UIState.isDropdownSidebar) isDropdownSidebar$: Observable<boolean>;

  @Select(UIState.currentTheme) theme$: Observable<string>;

  ui: UIStateModel;

  fontSizeKeys: string[] = Object.values(UIFontSize).filter(val => typeof val === 'string') as string[];

  fontSizeObject = getFontSizeObject(this.fontSizeKeys);

  get _trackFontSize() {
    return UIFontSize[this.ui.fontSize];
  }

  constructor(private store: Store) { }

  ngOnInit() {

    this.store
      .select(UIState)
      .pipe(takeUntil(this._destroy))
      .subscribe(ui => {
        this.ui = ui;
        this.updateGlobalFontSize();
      });

    this.checkSidebarFolded();
  }

  ngOnDestroy() {
    this._destroy.next();
    this._destroy.complete();
  }

  changeSidebarMode(change: NtSwitchChange<any>) {
    this.store.dispatch(
      new UIActions.ChangeSidebarMode(
        change.checked
          ? UISidebarMode.Dropdown
          : UISidebarMode.Default
      )
    );
  }

  changeFontSize(fontSize: UIFontSize, dropdown: NtDropdownComponent) {
    this.store.dispatch(new UIActions.ChangeFontSize(fontSize));
    dropdown.overlay.forceUpdatePosition();
  }

  toggleReactiveMode(change: NtSwitchChange<any>) {
    this.store
      .dispatch(new UIActions.ToggleReactiveMode(change.checked))
      .subscribe(() => this.checkSidebarFolded());
  }

  changeTheme(change: any) {
    this.store.dispatch(new UIActions.ChangeTheme(change.checked ? 'dark' : 'light'));
  }

  @HostListener('window:resize')
  onWindowResize() {
    this.checkSidebarFolded();
  }

  private checkSidebarFolded() {
    if (this.ui.reactiveMode) {
      const windowWidth = window.innerWidth || document.documentElement.clientWidth;
      const dropdownMode = windowWidth < 1280 ? UISidebarMode.Dropdown : UISidebarMode.Default;
      if (dropdownMode !== this.ui.reactiveSidebarMode) {
        this.store.dispatch(new UIActions.ChangeReactiveSidebarMode(dropdownMode));
      }
    }
  }

  private updateGlobalFontSize() {
    if (this.ui.reactiveMode) {
      document.body.parentElement.style.removeProperty('font-size');
    } else {
      document.body.parentElement.style.fontSize = `${this.ui.fontSize}px`;
    }
  }
}
